from setuptools import setup

def read_version():
    with open("VERSION", "r") as f:
        return f.read().strip()

setup(
    name="fezrs",
    version=read_version(),
    setup_requires=["setuptools", "setuptools_scm"],
    packages=["fezrs"],
    install_requires=[
        "numpy",
        "matplotlib",
        "scikit-image",
        "scikit-learn",
        "fastapi",
        "opencv",
    ],
    author="Mahdi Farmahinifarahani, Hooman Mirzaee, Mahdi Nedaee, Mohammad Hossein Kiani Fayz Abadi, Yoones Kiani Feyz Abadi, Erfan Karimzadehasl, Parsa Elmi",
    author_email="aradfarahani@aol.com",
    description="Feature Extraction and Zoning for Remote Sensing (FEZrs)",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/FEZtool-team/FEZrs",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from sklearn.cluster import KMeans
import matplotlib.pyplot as plt
from fastapi import HTTPException, status
from .file_handler import OpenFiles

class KMEANSCalculator:
    """K-Means clustering for remote sensing analysis."""

    def __init__(self, files: OpenFiles):
        if files.nir_band is None:
            raise ValueError("NIR band is required for KMeans clustering.")
        self.files = files
        self.nir_band_metadata = files.nir_metadata
        self.nir = files.nir_band
        self.kmeans = None

    def calculate(self, extra_params: dict):
        if not isinstance(extra_params, dict) or "n_clusters" not in extra_params or "random_state" not in extra_params:
            raise HTTPException(
                detail="Provide 'n_clusters' and 'random_state' as parameters.",
                status_code=status.HTTP_412_PRECONDITION_FAILED,
            )

        n_clusters = extra_params["n_clusters"]
        random_state = extra_params["random_state"]

        image_reshape = self.nir.reshape(
            self.nir_band_metadata["width"] * self.nir_band_metadata["height"], 1
        )

        kmeans = KMeans(n_clusters=n_clusters, random_state=random_state)
        kmeans.fit(image_reshape)

        clusterd_image = kmeans.cluster_centers_[kmeans.labels_].reshape(
            self.nir_band_metadata["height"], self.nir_band_metadata["width"]
        )

        self.kmeans = clusterd_image
        return self.kmeans

    def export(self, file_path: str, title: str):
        plt.figure(figsize=(10, 10))
        plt.title(title)
        plt.imshow(self.kmeans, cmap="viridis")
        plt.axis("off")
        plt.savefig(file_path)
        plt.close()



from pathlib import Path

import cv2
from matplotlib import pyplot as plt

from app.openrs.base import Base
from app.openrs.file_handler import OpenFiles


class GAUSSIANCalculator(Base):
    def __init__(self, files):
        # Check requirment bands
        if files.tif_file is None:
            raise Exception("Message")

        super().__init__(files)
        # Normalize the bands
        self.normalized_bands = files.get_normalize_bands()

        self.image = files.tif_file_metadata["image"]

    def calculate(self, extra_params):
        gaussian = cv2.GaussianBlur(self.image, (13, 13), 0)
        self.gaussian = gaussian

        return gaussian

    def export(self, file_path, title):
        plt.figure(figsize=(10, 10))
        plt.imshow(self.gaussian, cmap="gray")
        plt.title(title)
        plt.axis("off")
        plt.savefig(file_path)
        plt.close()


if __name__ == "__main__":
    tif_file = Path.cwd() / "app/openrs/data/pan_img.tif"
    calculator = GAUSSIANCalculator(OpenFiles(tif_file=tif_file))
    gaussian_image = calculator.calculate()
    plt.figure(figsize=(10, 10))
    plt.imshow(gaussian_image, cmap="gray")
    plt.title("Guassian Filter")
    plt.axis("off")
    plt.show()
